from . import mcda_method
from .codas import CODAS
from .edas import EDAS
from .mabac import MABAC
from .multimoora import MULTIMOORA
from .spotis import SPOTIS
from .topsis import TOPSIS
from .vikor import VIKOR
from .waspas import WASPAS